from fluxo.settings import AppSettings

from fluxo.fluxo_core.flow import Flow
from fluxo.fluxo_core.task import Task

from fluxo.fluxo_core.intervals import (
    Minutes,
    Hours,
    Days
)

__version__ = AppSettings.VERSION